<?php

namespace Zotlabs\Module;


use App;
use Zotlabs\Web\Controller;
use Zotlabs\Lib\PermissionDescription;
use Zotlabs\Web\HTTPSig;
use Zotlabs\Lib\Libzot;
use Zotlabs\Lib\PConfig;
use Zotlabs\Lib\Group;

require_once('include/items.php');
require_once('include/bbcode.php');
require_once('include/security.php');
require_once('include/conversation.php');
require_once('include/acl_selectors.php');
require_once('include/opengraph.php');

require_once('addon/forum/forum.php');
/**
 * @brief Channel Controller
 *
 */

class Forum extends Controller {

	private static $which='';

	function init() {

		if(in_array(substr($_GET['search'],0,1),[ '@', '!', '?']))	
			goaway('search' . '?f=&search=' . $_GET['search']);

		$which = null;
		if(argc() > 1)
			$which = argv(1);
		if(! $which) {
			if(local_channel()) {
				$channel = App::get_channel();
				if($channel && $channel['channel_address'])
				$which = $channel['channel_address'];
			}
		}
		if(! $which) {
			notice( t('You must be logged in to see this page.') . EOL );
			return;
		}

		self::$which=$which;
		$profile = 0;
		$channel = App::get_channel();

		if((local_channel()) && (argc() > 2) && (argv(2) === 'view')) {
			$which = $channel['channel_address'];
			$profile = argv(1);
		}

		$channel = channelx_by_nick($which);
		if(! $channel) {
			http_status_exit(404, 'Not found');
		}

		if((local_channel()) && (argc() > 2) && (argv(2) === 'view')) {
			$which = $channel['channel_address'];
			$profile = argv(1);
		}

		head_add_link( [ 
			'rel'   => 'alternate', 
			'type'  => 'application/atom+xml',
			'title' => t('Posts and comments'),
			'href'  => z_root() . '/feed/' . $which
		]);

		head_add_link( [ 
			'rel'   => 'alternate', 
			'type'  => 'application/atom+xml',
			'title' => t('Only posts'),
			'href'  => z_root() . '/feed/' . $which . '?f=&top=1'
		]);


		// Run profile_load() here to make sure the theme is set before
		// we start loading content
		profile_load($which,$profile);
		
		// Add Opengraph markup
		$mid = ((x($_REQUEST,'mid')) ? $_REQUEST['mid'] : '');
		if(strpos($mid,'b64.') === 0)
		    $mid = @base64url_decode(substr($mid,4));
		    
		if($mid)
		    $r = q("SELECT * FROM item WHERE mid = '%s' AND uid = %d AND item_private = 0 LIMIT 1",
		        dbesc($mid),
		        intval($channel['channel_id'])
		    );
		    
		opengraph_add_meta($r ? $r[0] : [], $channel);
	}

	function get($update = 0, $load = false) {

		if (argc() == 4 && argv(2)=='subscribe') {
			\ForumUtils::dosubscribe();
		}
		if (argc() == 4 && argv(2)=='unsubscribe') {
			\ForumUtils::dounsubscribe();
		}

		$noscript_content = get_config('system', 'noscript_content', '1');

                $category = $datequery = $datequery2 = $topic = '';

                $mid = ((x($_REQUEST, 'mid')) ? $_REQUEST['mid'] : '');

                if (strpos($mid, 'b64.') === 0)
                        $decoded = @base64url_decode(substr($mid, 4));
                if ($decoded)
                        $mid = $decoded;

                $datequery  = ((x($_GET, 'dend') && is_a_date_arg($_GET['dend'])) ? notags($_GET['dend']) : '');
                $datequery2 = ((x($_GET, 'dbegin') && is_a_date_arg($_GET['dbegin'])) ? notags($_GET['dbegin']) : '');

		if(observer_prohibited(true)) {
			return login();
		}

                $category = ((x($_REQUEST, 'cat')) ? $_REQUEST['cat'] : '');
                $hashtags = ((x($_REQUEST, 'tag')) ? $_REQUEST['tag'] : '');
                $order    = ((x($_GET, 'order')) ? notags($_GET['order']) : 'post');
                $search   = ((x($_GET, 'search')) ? $_GET['search'] : EMPTY_STR);
		$topic    = ((x($_REQUEST,'topic')) ? $_REQUEST['topic'] : '');

		$groups = [];

		$o = '';

		if($update) {
			// Ensure we've got a profile owner if updating.
			App::$profile['profile_uid'] = App::$profile_uid = $update;
		}

		$is_owner = (((local_channel()) && (App::$profile['profile_uid'] == local_channel())) ? true : false);

		$channel = App::get_channel();
		$observer = App::get_observer();
		$ob_hash = (($observer) ? $observer['xchan_hash'] : '');

		$perms = get_all_perms(App::$profile['profile_uid'],$ob_hash);

		if(! $perms['view_stream']) {
			// We may want to make the target of this redirect configurable
			if($perms['view_profile']) {
				notice( t('Insufficient permissions.  Request redirected to profile page.') . EOL);
				goaway (z_root() . "/profile/" . App::$profile['channel_address']);
			}
			notice( t('Permission denied.') . EOL);
			return;
		}


		if(! $update) {

			//nav_set_selected('Channel Home');

			// search terms header
			if($search) {
				$o .= replace_macros(get_markup_template("section_title.tpl"),array(
					'$title' => t('Search Results For:') . ' ' . htmlspecialchars($search, ENT_COMPAT,'UTF-8')
				));
			}

			/*
			if($channel && $is_owner) {
				$channel_acl = array(
					'allow_cid' => $channel['channel_allow_cid'],
					'allow_gid' => $channel['channel_allow_gid'],
					'deny_cid' => $channel['channel_deny_cid'],
					'deny_gid' => $channel['channel_deny_gid']
				);
			}
			else {
				$channel_acl = [ 'allow_cid' => '', 'allow_gid' => '', 'deny_cid' => '', 'deny_gid' => '' ];
			}
			*/

			$channel_acl = [ 'allow_cid' => '', 'allow_gid' => '', 'deny_cid' => '', 'deny_gid' => '' ];


			if($perms['post_wall']) {

				$x = array(
					'is_owner' => $is_owner,
					'allow_location' => ((($is_owner || $observer) && (intval(get_pconfig(App::$profile['profile_uid'],'system','use_browser_location')))) ? true : false),
					'default_location' => (($is_owner) ? App::$profile['channel_location'] : ''),
					'nickname' => App::$profile['channel_address'],
					'lockstate' => (((strlen(App::$profile['channel_allow_cid'])) || (strlen(App::$profile['channel_allow_gid'])) || (strlen(App::$profile['channel_deny_cid'])) || (strlen(App::$profile['channel_deny_gid']))) ? 'lock' : 'unlock'),
					'acl' => '',
					'permissions' => $channel_acl,
					'showacl' => '',
					'bang' => '',
					'visitor' => (($is_owner || $observer) ? true : false),
					'profile_uid' => App::$profile['profile_uid'],
					'editor_autocomplete' => true,
					'bbco_autocomplete' => 'bbcode',
					'bbcode' => true,
					'jotnets' => true,
					'reset' => t('Reset form')
				);

				$o .= "<a name='TOP'></a>";
				$o .= status_editor($a,$x,false,'Channel');
			}

		}


		/**
		 * Get permissions SQL - if $remote_contact is true, our remote user has been pre-verified and we already have fetched his/her groups
		 */
		 
		 $item_normal = " and item.item_hidden = 0 and item.item_type = 0 and item.item_deleted = 0
		    and item.item_unpublished = 0 and item.item_pending_remove = 0
		    and item.item_blocked = 0 ";
		 if (! $is_owner)
		    $item_normal .= "and item.item_delayed = 0 ";
		$item_normal_update = item_normal_update();
		$sql_extra = item_permissions_sql(App::$profile['profile_uid']);

		if(feature_enabled(App::$profile['profile_uid'], 'channel_list_mode') && (! $mid))
			$page_mode = 'list';
		else
			$page_mode = 'client';

		$abook_uids = " and abook.abook_channel = " . intval(App::$profile['profile_uid']) . " ";

		$simple_update = '';

		head_add_link([ 
			'rel'   => 'alternate',
			'type'  => 'application/json+oembed',
			'href'  => z_root() . '/oep?f=&url=' . urlencode(z_root() . '/' . App::$query_string),
			'title' => 'oembed'
		]);


		if ($order === 'post') {
			$ordering = 'created';
		} else {
			$ordering = 'commented';
		}


		$topic_sql = ($topic) ? "left join iconfig as tuid_t on (item.parent = tuid_t.iid and tuid_t.cat ='forum' and tuid_t.k='topicuuid' and tuid_t.v='$topic')" : "left join iconfig as tuid_t on (item.id = tuid_t.iid and tuid_t.cat ='forum' and tuid_t.k='topicuuid')";

		$itemspage = 10;
		$itemspage = get_pconfig(local_channel(),'system','itemspage',$itemspage);
		$itemspage = get_pconfig(App::$profile['profile_uid'],'system','itemspage',$itemspage);
		$itemspage = get_pconfig(App::$profile['profile_uid'],'forum','tocitems',$itemspage);

		App::set_pager_itemspage(((intval($itemspage)) ? $itemspage : 10));

		$pager_sql = sprintf(" LIMIT %d OFFSET %d ", intval(App::$pager['itemspage']), intval(App::$pager['start']));

		if(($mid) && ($noscript_content || $load)) {
			if($mid) {
					$r = q("SELECT parent AS item_id from item where mid like '%s' and uid = %d $item_normal
					AND item_wall = 1 $sql_extra limit 1",
					dbesc($mid . '%'),
					intval(App::$profile['profile_uid'])
				);
				if (! $r) {
					notice( t('Permission denied.') . EOL);
				}
			}
			else {
				$r = q("SELECT DISTINCT item.parent AS item_id, $ordering FROM item 
					left join abook on ( item.author_xchan = abook.abook_xchan $abook_uids )
					$topic_sql
					WHERE true and item.uid = %d $item_normal
					AND (abook.abook_blocked = 0 or abook.abook_flags is null)
					AND item.item_wall = 1 AND item.item_thread_top = 1
					$sql_extra $sql_extra2 
					ORDER BY $ordering DESC, item_id $pager_sql ",
					intval(App::$profile['profile_uid'])
				);
			}
		}
		else {
			$r = array();
		}

		if((! $update) && (! $load)) {

			if (!($decoded)) {
				$itemslimit = ($itemspage && !$topic) ? "limit $itemspage" : '';

				//Get enabled topics
				$topics = \ForumUtils::get_forumtopics(App::$profile_uid,true);

				//Cycle through enabled topics and create table

				$enabledtopics=[];
				$query = '';
				$results = [];
				foreach ($topics as $idx => $topicdata) {
					$enabled = isset($topicdata['enabled']) ? $topicdata['enabled'] : 0;

					if ($topic && $topic!=$topicdata['uuid']) {
						continue;
					}

					if (!$enabled && !$topic) {
						continue;
					}

					$topicdata['hashes']='';
					if (count($topicdata['hashtags'])) {
						$topicdata['hashes'] = '#' . implode(', #',$topicdata['hashtags']);
					}

					$restrictedread = (isset($topicdata['topic_allow_gid']) 
					    && $topicdata['topic_allow_gid'] != ''
					    && $topicdata['topic_allow_gid'] != "ChannelSettings");

					$subscribelink='';
					$subscribetext='';

					$ingroups=Group::containing(App::$profile_uid,get_observer_hash());
					$is_intopicallow = false;
					$grouphash=$topicdata['topic_allow_gid'];

					$grouprec = Group::rec_byhash(App::$profile_uid,$grouphash);
					if ($grouprec) {

						$gid = $grouprec['id'];

						if (in_array($gid,$ingroups)) {
							$is_intopicallow = true;
						}
					}

					if ( $restrictedread && !$is_intopicallow ) {
						if (!isset($topicdata['selfsubscribe']) || !$topicdata['selfsubscribe']) {
							continue;
						}


						//$subscribelink=z_root().'/forum/'.$channel['xchan_addr'].'/subscribe/'.$topicdata['uuid'];
						$subscribelink=z_root().'/forum/'.App::$profile['channel_address'].'/subscribe/'.$topicdata['uuid'];
						$subscribetext=t('Subscribe');

					} else if ($restrictedread) {
						if (isset($topicdata['selfsubscribe']) && $topicdata['selfsubscribe']) {
							//$subscribelink=z_root().'/forum/'.$channel['xchan_addr'].'/unsubscribe/'.$topicdata['uuid'];
							$subscribelink=z_root().'/forum/'.App::$profile['channel_address'].'/unsubscribe/'.$topicdata['uuid'];
							$subscribetext=t('Unsubscribe');
						}
					}

			                $observerhash=get_observer_hash();
                			$r=q("select abook_id from abook where abook_xchan='%s' and abook_channel=%d",$observerhash,App::$profile_uid);
					if ($r && $subscribelink) {
						$topicdata['subscribelink']="<a href='".$subscribelink."' class='forum-subscribelink'>".$subscribetext."</a>";
					} else {
						$topicdata['subscribelink']='';
					}

					$enabledtopics[$topicdata['uuid']] = $topicdata;

					$tuid = $topicdata['uuid'];
					$topic_sql2 = "inner join iconfig as tuid_t on (item.parent = tuid_t.iid and tuid_t.cat ='forum' and tuid_t.k='topicuuid' and tuid_t.v='$tuid')";

					$query = sprintf("SELECT DISTINCT item.id, title, body, llink, created, commented, item.id as item_id, '$tuid' as topicuuid
							from item
							$topic_sql2
							left join abook on (item.author_xchan = abook.abook_xchan $abook_uids )
							where true
							AND (abook.abook_blocked = 0 or abook.abook_flags is null)
							AND item.item_wall = 1 AND item.item_thread_top = 1
							and item.uid = %d $item_normal
							$sql_extra $sql_extra2 
							order by $ordering DESC
							$itemslimit
							",
							intval(App::$profile['profile_uid'])
							);

					$r = q($query);

					if ($r) {
						foreach ($r as $res) {

							$results[] = $res;

							}
					}
				}
				$r = $results;
/*
						//MYSQL > ver 8 can use: row_number() over (partition by iconfig.v) AS rnum
						//NOTE: MYSQL ONLY - need to create a POSTGRES compatible solution;
				$query2 = sprintf("SELECT * from (SELECT *,
						@row_number:=CASE WHEN @tuuid = topicuuid THEN @row_number + 1 ELSE 1 END AS rnum,
						@tuuid:=topicuuid
						FROM (SELECT DISTINCT item.id, title, body, llink, created, commented, item.id as item_id, tuid_t.v as topicuuid
						from item
						$topic_sql
						left join abook on (item.author_xchan = abook.abook_xchan $abook_uids )
						WHERE true
						AND (abook.abook_blocked = 0 or abook.abook_flags is null)
						AND item.item_wall = 1 AND item.item_thread_top = 1
						and item.uid = %d $item_normal
						$sql_extra $sql_extra2 
						) as resulttable
						ORDER BY topicuuid, $ordering DESC, item_id) as resulttable_2
						$itemslimit
						",
						intval(App::$profile['profile_uid'])
					);
				$r = q($query);
*/
				$itemsbytopic = [];

				foreach ($r as $item) {
					if (!isset($enabledtopics[$item['topicuuid']])) {
						continue;
					}

					$item['commented'] = datetime_convert('UTC',App::$timezone,$item['commented']);
					$itemsbytopic[$item['topicuuid']][] = $item;
				}


				$tocheaderbb = PConfig::Get(\App::$profile_uid,'forum','tocheaderbb','');

                		$tocheader = bbcode($tocheaderbb);

				$footer = "<div class='row'><div class='col-10'></div><div class='col-2'>";
				$footer .= "<a href='".z_root()."/settings/forum'>Settings</a>";
				$footer .="</div></div>";

				if (local_channel() != \App::$profile_uid) {
					$footer = '';
				}

				$o .= replace_macros(get_markup_template("toc.tpl",'addon/forum'), [
						'$topics' => $enabledtopics,
						'$tocheader' => $tocheader,
						'$itemsbytopic' => $itemsbytopic,
						'$forumurl' => z_root() . '/forum/' . self::$which . '',
						'$footer' => $footer
					]);

				$items=[];

			} else {

				if($r) {

					$parents_str = ids_to_querystr($r,'item_id');
	
					$r = q("SELECT item.*, item.id AS item_id
						FROM item
						WHERE item.uid = %d $item_normal
						AND item.parent IN ( %s )
						$sql_extra ",
						intval(App::$profile['profile_uid']),
						dbesc($parents_str)
					);

					xchan_query($r);
					$items = fetch_post_tags($r, true);
					$items = conv_sort($items,$ordering);
	
					if($load && $mid && (! count($items))) {
						// This will happen if we don't have sufficient permissions
						// to view the parent item (or the item itself if it is toplevel)
						notice( t('Permission denied.') . EOL);
					}

				} else {
					$items = array();
				}

				if($decoded)
					$mid = 'b64.' . base64url_encode($mid);

				// This is ugly, but we can't pass the profile_uid through the session to the ajax updater,
				// because browser prefetching might change it on us. We have to deliver it with the page.

				$maxheight = get_pconfig(App::$profile['profile_uid'],'system','channel_divmore_height');
				if(! $maxheight)
					$maxheight = 400;

				$o .= '<div id="live-channel"></div>' . "\r\n";
				$o .= "<script> var profile_uid = " . App::$profile['profile_uid']
					. "; var netargs = '?f='; var profile_page = " . App::$pager['page']
					. "; divmore_height = " . intval($maxheight) . ";</script>\r\n";
				//@TODO - update/modify template for searching by topic
				App::$page['htmlhead'] .= replace_macros(get_markup_template("build_query.tpl"),array(
					'$baseurl' => z_root(),
					'$pgtype' => 'forum',
					'$uid' => ((App::$profile['profile_uid']) ? App::$profile['profile_uid'] : '0'),
					'$gid' => '0',
					'$cid' => '0',
					'$cmin' => '(-1)',
					'$cmax' => '(-1)',
					'$star' => '0',
					'$liked' => '0',
					'$conv' => '0',
					'$spam' => '0',
					'$nouveau' => '0',
					'$wall' => '1',
					'$fh' => '0',
					'$dm' => '0',
					'$page' => ((App::$pager['page'] != 1) ? App::$pager['page'] : 1),
					'$search' => $search,
					'$xchan' => '',
					'$order' => (($order) ? urlencode($order) : ''),
					'$list' => ((x($_REQUEST,'list')) ? intval($_REQUEST['list']) : 0),
					'$file' => '',
					'$cats' => (($category) ? urlencode($category) : ''),
					'$tags' => (($hashtags) ? urlencode($hashtags) : ''),
					'$mid' => (($mid) ? urlencode($mid) : ''),
					'$verb' => '',
					'$net' => '',
					'$dend' => $datequery,
					'$dbegin' => $datequery2,
					'$conv_mode' => 'channel'
				));

			}
		}

		$mode = (($search) ? 'search' : 'channel');

		if (!$topic) {
			if($update) {
				$o .= conversation($items,$mode,$update,$page_mode);
			}
			else {
	
				$o .= '<noscript>';
				if($noscript_content) {
					$o .= conversation($items,$mode,$update,'traditional');
					$o .= alt_pager(count($items));
				}
				else {
					$o .= '<div class="section-content-warning-wrapper">' . t('You must enable javascript for your browser to be able to view this content.') . '</div>';
				}
				$o .= '</noscript>';
	
				$o .= conversation($items,$mode,$update,$page_mode);
	
				if ($mid && $items[0]['title'])
					App::$page['title'] = $items[0]['title'] . " - " . App::$page['title'];
	
			}

			if($mid || !($topic))
				$o .= '<div id="content-complete"></div>';
		}

		$_SESSION['loadtime'] = datetime_convert();

		return $o;
	}
}
