<?php

namespace Zotlabs\Module\Settings;

use Zotlabs\Web\Controller;
use Zotlabs\Lib\PConfig;
use Zotlabs\Lib\IConfig;
use Zotlabs\Lib\Group;
use Zotlabs\Extend\Hook;
use Zotlabs\Extend\Route;
use Zotlabs\Lib\Apps;
use Ramsey\Uuid\Uuid;

require_once("addon/forum/forum.php");
require_once("include/bbcode.php");
require_once("include/text.php");

class Forum extends Controller {
	function init () {

		$which = null;

		if(local_channel()) {
			$channel = App::get_channel();
			if($channel && $channel['channel_address'])
			$which = $channel['channel_address'];
		} else {
			notice( t('You must be logged in to see this page.') . EOL );
			return;
		}

		$profile = 0;
		$channel = App::get_channel();

		$channel = channelx_by_nick($which);
		if(! $channel) {
			http_status_exit(404, 'Not found');
		}

		// Run profile_load() here to make sure the theme is set before
		// we start loading content
		profile_load($which,$profile);
	}

	function post () {

		check_form_security_token_redirectOnErr('/settings/forum', 'settings');

		$indexes = $_REQUEST['forumtopic'];
		$titles = $_REQUEST['topictitle'];
		$topicsenabled = $_REQUEST['topicenabled'];
		$topichashtags = $_REQUEST['topichashtags'];
		$readgroups = $_REQUEST['readgroup'];
		$postgroups = $_REQUEST['postgroup'];
		$selfsubscribes = $_REQUEST['selfsubscribe'];

		$order = 0;
		$allhashtags = [];

		$forum_custom_groups = ['Anonymous','ChannelSettings'];
		foreach ($indexes as $index) {
			$new = 0;
			$order++;
			if (strpos($index,'new_') === 0 || $index==='') {
				$uuid = Uuid::uuid4();
				$new = 1;
			} else {
				$uuid = preg_replace('/[^a-zA-Z0-9\-\_]/','',$index);
			}

			$title = preg_replace('/[^a-zA-Z0-9\-\_\ ]/','',$titles[$index]);
			$enabled = $new ? 1 : intval($topicsenabled[$index]);
			$hashtags= preg_replace('/#+/','',$topichashtags[$index]);
			$hashtags= preg_split('/[\s,]+/',$hashtags);
			$hashtags= array_map('strtolower', $hashtags);
			$hashtags= array_diff($hashtags,$allhashtags);
			$allhashtags = array_merge($allhashtags,$hashtags);

			$readgroup = Group::rec_byhash(\App::$profile_uid,$readgroups[$index]) ? $readgroups[$index] : '';
			$postgroup = Group::rec_byhash(\App::$profile_uid,$postgroups[$index]) ? $postgroups[$index] : '';

			$forum_custom_groups = ['Anonymous','ChannelSettings'];

			$readgroup = in_array($readgroups[$index],$forum_custom_groups) ? $readgroups[$index] : $readgroup;
			$postgroup = in_array($postgroups[$index],$forum_custom_groups) ? $postgroups[$index] : $postgroup;

			$selfsubscribe = intval($selfsubscribes[$index]);
			$moremeta=[
				'topic_allow_gid' => $readgroup,
				'post_allow_gid' => $postgroup,
				'selfsubscribe' => $selfsubscribe
			];
		logger("META: ".json_encode($moremeta));	

			\ForumUtils::add_topic($uuid,$title,$order,$enabled,$hashtags,$moremeta);
		}

		\ForumUtils::save_forumtopics();

		if (isset($_REQUEST['tocitems']) && intval ($_REQUEST['tocitems'])) {
			PConfig::Set(\App::$profile_uid,'forum','tocitems',intval($_REQUEST['tocitems']));
		}

		if (isset($_REQUEST['tocheaderbb'])) {
			$tocheaderbb = $_REQUEST['tocheaderbb'];
			$tocheaderbb = cleanup_bbcode($tocheaderbb);
			PConfig::Set(\App::$profile_uid,'forum','tocheaderbb',$_REQUEST['tocheaderbb']);
		}

		goaway('/settings/forum');

	}

	function get () {

		$form_security_token = get_form_security_token('settings');

		head_add_js('/addon/forum/view/js/jquery-ui-1.12.1/jquery-ui.min.js',99);
		head_add_js('/addon/forum/view/js/jquery.ui.touch-punch.min.js',100);
		head_add_css('/addon/forum/view/js/jquery-ui-1.12.1/jquery-ui.min.css',99);

		//Set up current topics.

		$topics = \ForumUtils::get_forumtopics(\App::$profile_uid,true);

		$topiclist = '';

		foreach ($topics as $topic) {

			$topicuuid = $topic['uuid'];

			$hashtags = (isset($topic['hashtags']) && $topic['hashtags']) ? '#'.implode($topic['hashtags'],', #') : '';
			$readgroup = (isset($topic['topic_allow_gid']) && $topic['topic_allow_gid']) ? $topic['topic_allow_gid'] : '';
		
			$readgroups = self::group_select(\App::$profile_uid,$readgroup,'readgroup['.$topicuuid.']',t('Allowed to read:'));

			$postgroup = (isset($topic['post_allow_gid']) && $topic['post_allow_gid']) ? $topic['post_allow_gid'] : '';
		
			$postgroups = self::group_select(\App::$profile_uid,$postgroup,'postgroup['.$topicuuid.']',t('Allowed to post:'),true);

			$selfsubscribe = (isset($topic['selfsubscribe']) && $topic['selfsubscribe']) ? $topic['selfsubscribe'] : '';

			$templatevars = [
				'$uuid' => $topic['uuid'],
				'$topic' => $topic['name'],
				'$enabled' => $topic['enabled'],
				'$hashtags' => $hashtags,
				'$readgroup' => $readgroups,
				'$postgroup' => $postgroups,
				'$selfsubscribe' => $selfsubscribe

			];

			$topiclist .= replace_macros(get_markup_template('edittopics_textinput.tpl','addon/forum'),$templatevars);

		}

			$topicuuid = "' + itemidx + '";

			$readgroups = self::group_select(\App::$profile_uid,'',"readgroup[".$topicuuid."]",t('Allowed to read:'));
			$postgroups = self::group_select(\App::$profile_uid,'',"postgroup[".$topicuuid."]",t('Allowed to post:'),true);
		
			$templatevars = [
				'$uuid' => "' + itemidx + '",
				'$topic' => '',
				'$enabled' => 0,
				'$hashtags' => '',
				'$readgroup' => $readgroups,
				'$postgroup' => $postgroups,
				'$selfsubscribe' => ''

			];
			$addtemplate = str_replace(array("\n","\r"),'',replace_macros(get_markup_template('edittopics_textinput.tpl','addon/forum'),$templatevars));
	
		$tocitems = PConfig::Get(\App::$profile_uid,'forum','tocitems',10);	
		$tocsetting = replace_macros(get_markup_template('field_input.tpl',''),['field'=>['tocitems','TOC Item Count',$tocitems,false,false,'Number of items to display under each topic in the Table of Contents page.']]);

		$tocheaderbb = PConfig::Get(\App::$profile_uid,'forum','tocheaderbb','');

		$tocheader = bbcode($tocheaderbb);

		$tocheaderbbfield = replace_macros(get_markup_template('field_textarea.tpl',''),['field'=>['tocheaderbb','TOC Header Edit Window',$tocheaderbb,'BBCode Allowed']]);

		$taggeddeliveryenabled = '';

		$taggeddeliveryxchans = q("select xchan from abconfig where chan = %d and cat = 'my_perms' and k = 'tag_deliver' and v = '1'", \App::$profile_uid );

		if ($taggeddeliveryxchans) {
			$taggeddeliveryenabled = true;
		}


		
		$templatevars = [
			'$pubcrawlinstalled' => Apps::addon_app_installed($uid,'pubcrawl'),
			'$form_security_token' => $form_security_token,
			'$topiclist' => $topiclist,
			'$addtemplate' => $addtemplate,
			'$TOCSETTING' => $tocsetting,
			'$tocheader' => $tocheader,
			'$tocheaderbbfield' => $tocheaderbbfield,
			'$taggeddeliveryenabled' => $taggeddeliveryenabled
		];

		$o .= replace_macros(get_markup_template('edittopics.tpl','addon/forum'),$templatevars);

		return $o;

	}

	private function group_select($uid,$group = '',$varname,$label,$includeanon=false) {

		$grps = array();
        	$o = '';

        	$r = q("SELECT * FROM pgrp WHERE deleted = 0 AND uid = %d ORDER BY gname ASC",
                	intval($uid)
        	);
        	$grps[] = array('name' => 'Use Channel Connection Settings', 'hash' => 'ChannelSettings', 'selected' => '');

		if ($includeanon) {
        		//$grps[] = array('name' => 'Anonymous', 'hash' => 'Anonymous', 'selected' => '');
			//NOT YET IMPLEMENTED
		}

        	if($r) {
                	foreach($r as $rr) {
                        	$grps[] = array('name' => $rr['gname'], 'id' => $rr['hash'], 'selected' => (($group == $rr['hash']) ? 'true' : ''));
                	}
	
        	}
        	logger('mini_group_select: ' . print_r($grps,true), LOGGER_DATA);
	
        	$o = replace_macros(get_markup_template('group_selection.tpl','addon/forum'), array(
                	'$label' => $label,
                	'$groups' => $grps,
			'$varname' => $varname
        	));
        	return $o;
	}
}
