<?php
/**
 * Name: forum
 * Description: A multiuser topical forum system that allows posting via direct messaging or wall posts.
 * Version: 2.0.1
 * Author: DM42.Net, LLC
 * MinVersion: 5.2.0
 */
 // Maintainer: devhubzilla@dm42.net

use Zotlabs\Lib\PConfig;
use Zotlabs\Lib\IConfig;
use Zotlabs\Extend\Hook;
use Zotlabs\Extend\Route;
use Zotlabs\Lib\Apps;
use Zotlabs\Lib\Group;
use Ramsey\Uuid\Uuid;
use Zotlabs\Lib\System;
use Zotlabs\Daemon\Master;

require_once('include/permissions.php');

function forum_load() {
	if (System::get_platform_name() == 'hubzilla') {
		Hook::register('page_header', 'addon/forum/forum.php', 'ForumUtils::page_header',1,527);
		Hook::register('item_store_before', 'addon/forum/forum.php', 'ForumUtils::hz_forum_item_store_before',1,527);
		Hook::register('post_local', 'addon/forum/forum.php', 'ForumUtils::post_local',1,527);
		Hook::register('tagged', 'addon/forum/forum.php', 'ForumUtils::hz_tagged',1,527);
		Hook::register('init_1', 'addon/forum/forum.php', 'ForumUtils::updatejottpl',1,527);
		Hook::register('bbcode', 'addon/forum/forum.php', 'ForumUtils::bbcode',1,527);
		Hook::register('channel_apps', 'addon/forum/forum.php', 'ForumUtils::channel_apps',1,527);

		Hook::register('jot_tpl_filter', 'addon/forum/forum.php', 'ForumUtils::customjotheaders',1,527);
		Route::register('addon/forum/Settings/SystemSettings.php','admin/addons/forum');
		Route::register('addon/forum/Settings/ChannelSettings.php','settings/forum');
		Route::register('addon/forum/Mod_forum.php','forum');
	} else {
		notice(t('INCOMPATIBLE PLATFORM') . EOL);
	}
}

function forum_unload() {
	Hook::unregister_by_file('addon/forum/forum.php');
	Route::unregister_by_file('addon/forum/Settings/SystemSettings.php');
	Route::unregister_by_file('addon/forum/Settings/ChannelSettings.php');
	Route::unregister_by_file('addon/forum/Mod_forum.php');
}

class ForumUtils {

	private static $in_forum_delivery = 0;
	private static $item_pre_post = null;

	public static $topiclist = null;
	public static $topictree = null;

	public static function page_header(&$header) {
		head_add_css('/addon/forum/view/css/forum.css');
	}

	private static function maybejson ($value,$options=0) {

		if ($value!=null) {
			if (!is_array($value)) {
				$decoded=json_decode($value,true);
			}
		} else {
			return null;
		}

		if (is_array($value) || json_last_error() != JSON_ERROR_NONE) {
			$encoded = json_encode($value,$options);
			return ($encoded);
		} else {
			return ($value);
		}
	}

	private static function maybeunjson ($value) {

		if (is_array($value)) {
			return $value;
		}

		if ($value!=null) {
			$decoded=json_decode($value,true);
		} else {
			return null;
		}

		if (json_last_error() == JSON_ERROR_NONE) {
			return ($decoded);
		} else {
			return ($value);
		}
	}

	protected static function sort_topiclist() {

		$forumtopics = self::$topiclist;

		if (!$forumtopics || !is_array($forumtopics)) {
			self::$topiclist = [];
		}

		usort($forumtopics,function($a,$b) {
			$aorder = (isset($a['order']) || $a['order'] === 0) ? $a['order'] : 1;
			$border = (isset($b['order']) || $b['order'] === 0) ? $b['order'] : 1;
			if (intval(@$aorder) == intval(@$border)) {
				return 0;
			}

			$ret = (intval(@$aorder) > intval(@$border)) ? 1 : -1;
			return $ret;
		});

		self::$topiclist=$forumtopics;

	}

	public static function get_forumtopics($uid,$reset=false) {

		if (!$reset && is_array(self::$topiclist) && count(self::$topiclist)) {
			return self::$topiclist;
		}

		$forumtopics = PConfig::Get($uid,'forum','topiclist','');

		$forumtopics = self::maybeunjson($forumtopics);

		if (!$forumtopics || !is_array($forumtopics)) {
			return [];
		}


		self::$topiclist = $forumtopics;
		self::sort_topiclist();

		$finaltopics = [];

		foreach (self::$topiclist as $ft) {
			$finaltopics[$ft['uuid']] = $ft;
		}

		self::$topiclist = $finaltopics;

		return self::$topiclist;
	}

	public static function save_forumtopics() {

		self::sort_topiclist();
		PConfig::Set(local_channel(),'forum','topiclist',self::maybejson(self::$topiclist));
	}

	public static function add_topic($uuid,$name,$order,$enabled=1,$hashtags=[],$moremeta=[]) {

		$curlist = self::$topiclist;
		if (!is_array($curlist)) {
			$curlist = [];
		}

		$newitem = [
				'uuid' => $uuid,
				'name' => $name,
				'order' => $order,
				'enabled' => $enabled,
				'hashtags' => $hashtags
			];

		foreach ($moremeta as $idx=>$meta) {
			$newitem[$idx]=$meta;
		}

		$curlist[] = $newitem;

		self::$topiclist = $curlist;
	}

	public static function updatejottpl() {
		if (version_compare(STD_VERSION,'5.2.0','>=') && version_compare(STD_VERSION,'5.3.4','<')) {
			\App::$override_markup_templates["ROOT"]["jot.tpl"]=Array("root"=>"addon/forum","file"=>"jot.tpl");
		}
	}

	public static function customjotheaders(&$templatevars) {

		$profileid = App::$profile_uid;

		if (!Apps::addon_app_installed(App::$profile_uid,'forum')) {
			return;
		}

		if (argv(0) != 'forum') {
			return;
		}

		$newvars=$templatevars;

		$default = null;

		if ($_REQUEST['forumtopic']) {
			$default = preg_replace('/[^a-zA-Z0-9\-\_]/','',$_REQUEST['forumtopic']);
		}

		$topics = self::get_forumtopics($profileid);

		foreach ($topics as $topic) {
			if (!$topic['enabled']) {
				continue;
			}
			$uuid = $topic['uuid'];

			if (isset($topic['post_allow_gid']) 
				&& $topic['post_allow_gid'] != ''
				&& $topic['post_allow_gid'] != 'ChannelSettings'
				) {
				if (!self::check_hashingroup (App::$profile_uid,get_observer_hash(),$topic['post_allow_gid'])) {
					continue;
				}
			}
			$topicselect[$uuid] = $topic['name'];
		}

		if (argv(0) == 'editpost') {
			$post_id = ((argc() > 1) ? intval(argv(1)) : 0);

			if(! $post_id) {
				notice( t('Item not found') . EOL);
				return;
			}

			$itm = q("SELECT * FROM item WHERE id = %d AND ( owner_xchan = '%s' OR author_xchan = '%s' ) LIMIT 1",
				intval($post_id),
				dbesc(get_observer_hash()),
				dbesc(get_observer_hash())
			);

			if ($itm['item_type'] != ITEM_TYPE_POST) {
				return;
			}
			$default = IConfig::Get($post_id, "forum", "topicuuid", $default);
		}

		$default = $default ? $default : PConfig::Get(App::$profile_uid,'forum','defaulttopic','');

		$topicselect = replace_macros(get_markup_template('field_select.tpl'),['$field'=>['forumtopic',t('Topic'),$default,'',$topicselect]]);

		$customheaders = '<div>'.$topicselect.'</div>';

		$newvars['customjotheaders'] .= $customheaders;

		$templatevars = $newvars;
	}

	static public function hz_tagged($data) {

		/*
                        $data = [
                                        'channel_id' => $uid,
                                        'item' => $item,
                                        'body' => $body
                        ];

		*/

		$uid = $data['channel_id'];
		$item = $data['item'];
		$datarray = $item;
		$arr = $datarray;
		$topicuuid='';

		$ownerchan = channelx_by_n($uid);
		$is_owner = (($ownerchan['channel_hash'] == $item['author_xchan']) ? true : false);

		if (!Apps::addon_app_installed($uid,'forum')) {
			return;
		}

		$u = q("select * from channel left join xchan on channel_hash = xchan_hash where channel_id = %d limit 1",
			intval($uid)
		);
		if(! $u) {
			return;
		}

		$forumtag = IConfig::Get($item,'forum','seen:'.$u['channel_hash'],false);
		if ($forumtag) {
			//Somewhere, we've touched this. Don't forward again.
			logger("Possible loop detected. Exiting forum posting.");
			return;
		}

		$user=$u[0];

		$channel=$user;
		/// INTEROP NOTE: POSSIBLE HUBZILLA SPECIFIC
		if (intval($arr['item_type']) != ITEM_TYPE_POST) {
			return;
		}

		$item = $arr;

		$is_comment=false;
        	if((($item['parent_mid']) && $item['parent_mid'] != $item['mid'])) {
                	$is_comment=true;
		}

		// Don't let anybody set these, either intentionally or accidentally

		if(array_key_exists('id',$item))
			unset($item['id']);
		if(array_key_exists('parent',$item))
			unset($item['parent']);

		$topiclist = self::get_forumtopics($uid);


		$edit = (($item['edited'] != $item['created']) || $item['item_deleted']);
		$olditem = [];
		$topicuuid=null;
		if ($edit) {
			$r = q("select * from item where uid = %d and resource_id = '%s' and source_xchan = '%s' and resource_type = 'group_item' limit 1",
				intval($channel['channel_id']),
				dbesc($item['mid']),
				dbesc($item['author_xchan'])
			);
			$olditem=$r[0];
		}

		if (!$is_comment) {
			if ($olditem) {
				$oldtopic = IConfig::Get($olditem['id'],'forum','topicuuid','');
				$topicuuid = $oldtopic;
			}

			if (!$topicuuid) {
				$topicuuid    = ((x($_REQUEST,'forumtopic')) ? $_REQUEST['forumtopic'] : '');
				$topicuuid = preg_replace('/[^a-zA-Z0-9\-\_]/','',$topicuuid);
			}

			if (!$olditem && !isset($topiclist[$topicuuid])) {
				$topicuuid = false;
			}

			if (!$topicuuid) {

				$htagterms = get_terms_oftype($arr['term'],[TERM_HASHTAG]);
				$hashtags=[];	

				foreach ($htagterms as $term) {
					$hashtags[] = $term['term'];
				}

				$hashtags = array_map('strtolower',$hashtags);

				foreach ($topiclist as $topic) {
					$topichashtag = isset($topic['hashtags']) ? array_intersect($topic['hashtags'],$hashtags) : false;
					if ($topichashtag) {
						$topicuuid=$topic['uuid'];
						break;
					}
				}
			}

			if (!$topicuuid) {
				return;
			}

			if (!isset($topiclist[$topicuuid])) {
				return;
			}
		}

		if ($is_comment) {
			$threadtopmid = $item['parent_mid'];
			$itemmid = $item['mid'];

			while ($itemmid !== $threadtopmid) {
				$r = q("select parent_mid,mid from item where uid = %d and parent_mid = '%s' and resource_type = 'group_item' limit 1",
					intval($channel['channel_id']),
					$parentmid
				);

				if (!$r) {
					break;
				}
				$threadtopmid = $r[0]['parent_mid'];
				$itemmid = $r[0]['mid'];
			}

			//Check for parent as 'group_item'. Bail if it doesn't exist.
			$r = q("select * from item where uid = %d and mid = '%s' and resource_type = 'group_item' limit 1",
				intval($channel['channel_id']),
				dbesc($threadtopmid)
			);

			if (!$r) {
				return;
			}

			$threadtopitem = $r[0];

			$item['allow_cid'] = $threadtopitem['allow_cid'];
			$item['allow_gid'] = $threadtopitem['allow_gid'];
			$item['deny_cid'] = $threadtopitem['deny_cid'];
			$item['deny_gid'] = $threadtopitem['deny_gid'];
			$item['parent'] = $threadtopitem['id'];

			$topicuuid = IConfig::Get($threadtopitem['id'],'forum','topicuuid');
			
		} else {
			$topicdata = $topiclist[$topicuuid];

			$topicallowcid = isset($topicdata['topic_allow_cid']) ? $topicdata['topic_allow_cid'] : '';
			$topicallowgid = isset($topicdata['topic_allow_gid']) ? $topicdata['topic_allow_gid'] : '';
			$topicallowgid = ($topicallowgid == "ChannelSettings") ? '' : $topicallowgid;
			$topicdenycid = isset($topicdata['topic_deny_cid']) ? $topicdata['topic_deny_cid'] : '';
			$topicdenygid = isset($topicdata['topic_deny_gid']) ? $topicdata['topic_deny_gid'] : '';


			$post_allow_gid=isset($topicdata['post_allow_gid']) ? $topicdata['post_allow_gid'] : '';
/*
			if (!self::check_hashingroup ($channel['channel_id'],$item['author_xchan'],$post_allow_gid)) {
				//Not allowed to post
				return;
			}
*/

			if ($topicallowcid || $topicallowgid || $topicdenycid || $topicdenygid) {
				$item['allow_cid'] = $topicallowcid;
				$item['allow_gid'] = $topicallowgid;
				$item['deny_cid'] = $topicdenycid;
				$item['deny_gid'] = $topicdenygid;
			} else {
				$item['allow_cid']=$channel['channel_allow_cid'];
				$item['allow_gid']=$channel['channel_allow_gid'];
				$item['deny_cid']=$channel['channel_deny_cid'];
				$item['deny_gid']=$channel['channel_deny_gid'];
			}

		}

	/**
		MODIFIED FROM: Hubzilla (v 5.2.1) - include/items.php ( start_delivery_chain() )
		GIT HASH: 254e30bea173e350a24756cbd8cf4acdfe32933e
	 */

	/**
		2021-02-08 - DISCOVERY: Forum comments are not forwarded to activitypub via
				pubcrawl from ZOT instances because iconfig does not contain
				rawmsg.  Since we cannot sign messages for forum members
				on other hubs, we will rewrite all posts - including comments.
	*/


		//It wasn't a comment, so set item_thread_top if it isn't already set
        	if(!$is_comment && ! array_key_exists('item_thread_top',$item) )
                	$item['item_thread_top'] = 1;

		if      ( !$is_comment && (
				!(intval($item['item_private'])) === 2 || 
				($item['author_xchan'] === $channel['channel_hash'] &&
			    	    !isset($_REQUEST['forumtopic']))
			)) {
			return;
		}

		if(! $item['uid']) {
			logger('no uid');
			return;
		}


		$arr = [];

		if ($olditem) {

			if ($olditem) {
				if (intval($item['item_deleted'])) {
					drop_item($r[0]['id'], false, DROPITEM_PHASE1);
					Master::Summon([ 'Notifier', 'drop', $r[0]['id'] ]);
					return;
				}
				$arr['id'] = intval($r[0]['id']);
				$arr['parent'] = intval($r[0]['parent']);
				$arr['uuid'] = $r[0]['uuid'];
				$arr['mid'] = $r[0]['mid'];
				$arr['parent_mid'] = $r[0]['parent_mid'];
				$arr['edited'] = ($item['edited']) ? $item['edited'] : datetime_convert();
				$arr['created'] = ($item['created']) ? $item['created'] : datetime_convert();
				$arr['changed'] = ($item['changed']) ? $item['changed'] : datetime_convert();
				//Yes, it's $item here because the actually assignment to $arr happens later.
				$item['allow_cid'] = $olditem['allow_cid'];
				$item['allow_gid'] = $olditem['allow_gid'];
				$item['deny_cid'] = $olditem['deny_cid'];
				$item['deny_gid'] = $olditem['deny_gid'];
			}
			else {
				logger('Forum post not found: ' . $item['mid']);
				return;
			}

		}
		else {
			$arr['uuid'] = item_message_id();
			$arr['mid'] = z_root() . '/activity/' . $arr['uuid'];
			if (!$is_comment) {
				$arr['parent_mid'] = $arr['mid'];
			} else {
				$arr['parent_mid'] = $item['parent_mid'];
			}
		}

		$arr['aid'] = $channel['channel_account_id'];
		$arr['uid'] = $channel['channel_id'];

		// WARNING: the presence of both source_xchan and non-zero item_uplink here will cause a delivery loop

		$arr['item_uplink']  = 0;
		
		$arr['source_xchan'] = $item['owner_xchan'];
		$arr['resource_id'] = $item['mid'];
		$arr['resource_type'] = 'group_item';

		IConfig::Set($arr, "forum", "topicuuid", $topicuuid, true);
		IConfig::Set($item,'forum','seen:'.$u['channel_hash'],'1',true);

		$arr['meta'] = encode_item_meta($arr['iconfig']);

		$arr['item_private'] = (($arr['allow_cid'] || $arr['allow_gid']
		|| $arr['deny_cid'] || $arr['deny_gid']) ? 1 : 0);

		$arr['item_origin'] = 1;

		$arr['llink'] = z_root() . '/display/' . gen_link_id($arr['mid']);
		$arr['plink'] = $arr['llink'];
		$originplink = $item['plink'] ? $item['plink'] : $arr['plink'];


		$author = q("select * from xchan where xchan_hash = '%s'",$item['author_xchan']);

		if ($author) {
		  $author = $author[0];
		} else {
		  //No author?  No post/comment
		  return;
		}

		$topicinfo = $topiclist[$topicuuid];

		$topicinfo['topicurl'] = z_root().'/forum/'.$channel['channel_address'].'?topic='.$topicuuid;
		$forumurl = z_root().'/forum/'.$channel['channel_address'];

		$action = ($is_comment) ? t('Reply by') : t('Post by');

		$topicinfo['hashlist'] = (isset($topicinfo['hashtags']) && $topicinfo['hashtags']) ? '#'.implode($topicinfo['hashtags'],', #') : '';

		$item['mid'] = ($item['mid']) ? $item['mid'] : $arr['mid'];

		$bb = replace_macros(get_markup_template('forumpublish.bb.tpl','addon/forum'),[
			'$item' => $item,
			'$arr' => $arr,
			'$author' => $author,
			'$topic' => $topicinfo,
			'$forumurl' => $forumurl,
			'$channel' => $channel,
			'$hostname' => \App::get_hostname(),
			'$action' => $action,
			'$originplink' => $originplink
			]);

		$arr['body'] = $bb;
		$arr['title'] = $item['title'];
		$arr['term'] = $item['term'];

		$arr['author_xchan'] = $channel['channel_hash'];
		$arr['owner_xchan']  = $channel['channel_hash'];
		$arr['route'] = '';
		//$arr['item_relay'] = 1;
		//$arr['edited'] = datetime_convert();
		$arr['edited'] = ($item['edited']) ? $item['edited'] : datetime_convert();
		$arr['created'] = ($item['created']) ? $item['created'] : datetime_convert();

		$arr['obj_type'] = $item['obj_type'];
		$arr['verb'] = ACTIVITY_POST;

		$arr['allow_cid'] = $item['allow_cid'];
		$arr['allow_gid'] = $item['allow_gid'];
		$arr['deny_cid']  = $item['deny_cid'];
		$arr['deny_gid']  = $item['deny_gid'];
		$arr['comment_policy'] = $item['comment_policy'];

		if ($edit) {
			logger("Update edited post");
			unset($arr['meta']);
			$updated = item_store_update($arr,$data['allow_exec'],false);
			if ($updated['success']) {
				$post_id = $updated['item']['id'];
				$arr=$updated['item'];
				$topic = IConfig::Get($arr,'forum','topicuuid',null);
				if ($topic) {	
					Master::Summon([ 'Notifier','tgroup',$post_id]);
				}
			} else {
				logger("Update failed for item.");
			}
			return;
		}

		if($is_comment) {
			q("UPDATE item set commented = '%s', changed = '%s' WHERE id = %d",
                        	dbesc($arr['commented']),
                        	dbesc($arr['changed']),
                        	intval($threadtopitem['id'])
                	);
		} else {
			$arr['item_wall'] = 1;
			$arr['item_thread_top'] = 0;
			$canpost = self::hash_canpost($uid,$item['author_xchan'],$topicuuid);

			if ( (!$is_owner) && (!$canpost)) {

				//User does not appear in group allowed to post.
				$arr['allow_cid']=perms2str([ $item['author_xchan'] ]);
				$arr['allow_gid']=perms2str([$topicinfo['post_moderators_gid']]);
				$arr['allow_gid']='';
				$arr['parent_mid']=$item['parent_mid'];
				$arr['parent']=$item['parent'];

				$postnotallowedmsg = PConfig::Get($uid,'forum','postnotallowedmsg',t('You do not have permission to post to this topic.'));
				$arr['body'] = $postnotallowedmsg;
				$edit=false;
				logger("POST REJECTED - no permission");
			}

			if (isset($topicinfo['topic_allow_gid']) && $topicinfo['topic_allow_gid']!='ChannelSettings') {
				$allow_cid_arr = expand_acl($item['allow_cid']);
				$allow_gid_arr = expand_acl($item['allow_gid']);
				$allow_cid_arr[] = $item['author_xchan'];
				$arr['allow_cid']=perms2str($allow_cid_arr);

				$allow_gid_arr[] = $topicinfo['topic_allow_gid'];
				$arr['allow_gid']=perms2str($allow_gid_arr);

			}
		}

		IConfig::Set($arr, "forum", "topicuuid", $topicuuid, true);

		$post = item_store($arr,$data['allow_exec'],false);
		if ($post['success']) {
			$post_id = $post['item']['id'];
			$arr=$post['item'];
			$topic = IConfig::Get($arr,'forum','topicuuid',null);
			if ($topic) {	
				Master::Summon([ 'Notifier','tgroup',$post_id]);
			}
			logger("New forum post: ".$post_id);
		}

		return;
	}

	public static function dosubscribe() {

		if (!Apps::addon_app_installed(App::$profile_uid,'forum')) {
			return;
		}

		if (!get_observer_hash()) {
			notice("You must be an authenticated from a compatible server on the network to subscribe.".EOT);
			return;
		}

		$topics = \ForumUtils::get_forumtopics(App::$profile_uid,true);
		$subscribetopic = argv(3);

		if (!isset($topics[$subscribetopic])) {
			notice("Invalid topic.");
			return;
		}

		$topic = $topics[$subscribetopic];
		if (!isset($topic['selfsubscribe']) || intval($topic['selfsubscribe']) != 1) {
			notice("Cannot subscribe to this topic.");
			return;
		}

          	$grouphash = $topic['topic_allow_gid'];

          	$grouprecord = group_rec_byhash(App::$profile_uid,$grouphash);
          	if (!$grouprecord) {
			notice("An error occurred.  You are not subscribed.");
			return;
		}

          	$groupname = $grouprecord["gname"];

          	$r=group_add_member(App::$profile_uid,$groupname,get_observer_hash());

          	if (!$r) {
			logger("Add to group failed");
			notice("An error occurred.  We were unable to subscribe you at this time.  Please try again later.");
                	}

		goaway(z_root().'/forum/'.App::$profile['channel_address']);

	}

	public static function dounsubscribe() {

		if (!Apps::addon_app_installed(App::$profile_uid,'forum')) {
			return;
		}

		if (!get_observer_hash()) {
			notice("You must be an authenticated from a compatible server on the network to subscribe.".EOT);
			return;
		}

		$topics = \ForumUtils::get_forumtopics(App::$profile_uid,true);
		$subscribetopic = argv(3);

		if (!isset($topics[$subscribetopic])) {
			notice("Invalid topic.");
			return;
		}

		$topic = $topics[$subscribetopic];
		if (!isset($topic['selfsubscribe']) || intval($topic['selfsubscribe']) != 1) {
			notice("Cannot subscribe to this topic.");
			return;
		}

          	$grouphash = $topic['topic_allow_gid'];

          	$grouprecord = group_rec_byhash(App::$profile_uid,$grouphash);
          	if (!$grouprecord) {
			notice("An error occurred.  You are not subscribed.");
			return;
		}

          	$groupname = $grouprecord["gname"];

          	$r=group_rmv_member(App::$profile_uid,$groupname,get_observer_hash());

          	if (!$r) {
			logger("Remove from group failed");
			notice("An error occurred.  You are still subscribed.  The administrators have been notified so that they can investigate and manually remove you.  This may take some time.  Please be patient.");

			$arr=[];
			$channel = channelx_by_n(App::$profile_uid);
			$requestor = channelx_by_hash(get_observer_hash());
			$arr['author_xchan'] = $channel['channel_hash'];
			$arr['owner_xchan'] = $channel['channel_hash'];
			$arr['allow_cid'] = perms2str([$channel['channel_hash']]);
			$arr['allow_gid'] = ''; //@TODO: Add "moderator group" to config and create moderation tools
			$arr['aid'] = $channel['channel_account_id'];
                	$arr['uid'] = $channel['channel_id'];

			$arr['title'] = "[".$channel['xchan_name']."] ".t("Error:")." ".t("Could not automatically unsubscribe user from topic.");

			$template = get_markup_template('error.unsubscribe.bb.tpl','addon/forum');
			$data = [
				'$requestor' => $requestor,
				'$channel' => $channel,
				'$groupname' => $groupname
			];

			$arr['body'] = replace_macros($template,$data);

			$post = item_store($arr,$data['allow_exec'],true);
                	if (!$post['success']) {
				$errtxt = "\n============";
				$errtxt .= $arr['title'];
				$errtxt .= "\n============";
				$errtxt .= "\n".$arr['body'];
				$errtxt .= "\n============";
				$errtxt .= "\nChannel: ".$channel['xchan_url'];
				$errtxt .= "\n============";
                        	logger("Error sending error: ".$errtxt);
                	}

		}
		goaway(z_root().'/forum/'.App::$profile['channel_address']);
	}

	public static function bbcode(&$Text) {

		if (!Apps::addon_app_installed(App::$profile_uid,'forum')) {
			return;
		}

		$parsed = $Text;

		$observer = App::get_observer();
		$uid = App::$profile_uid;

		$path='';
		for ($i=0;$i < argc();$i++) {
			$path.='/'.argv($i);
		}

		$loginhtml = "<div style='display: inline;'><form method=get action='".$path."'>Enter your channel address: <input type=text size=25 name='zid' /><button type=submit value='".t('Login')."'>".t('Login')."</button></form></div>";
		$parsed = (!$observer) ? preg_replace("/\[forum\.login\]/ism", $loginhtml, $parsed) : preg_replace("/\[forum\.login\]/ism", '', $parsed);

		$connected=0;
		$observerhash=get_observer_hash();

		$observerself = 0;

		$channel = channelx_by_n($uid);

		if ($channel['channel_hash'] == $observerhash) {
			$connected = 1;
			$observerself=1;
		}

		$r=q("select abook_id from abook where abook_xchan='%s' and abook_channel=%d",$observerhash,$uid);
		if (!$connected && count($r)) {
			$connected=1;
		}

                $connect_url = rconnect_url(App::$profile_uid,$observerhash);
                $connect = (($connect_url) ? t('Connect') : '');
                if($connect_url)
                        $connect_url = sprintf($connect_url,urlencode(channel_reddress(\App::$profile)));

		$connecthtml = '<div class="connect-btn-wrapper"><a href="'.$connect_url.'" target="_blank" class="btn btn-block btn-success btn-sm"><i class="fa fa-plus"></i> '.$connect.'</a></div>';

		$parsed = (!$connected && $connect) ? preg_replace("/\[forum\.connect\]/ism", $connecthtml, $parsed) : preg_replace("/\[forum\.connect\]/ism", '', $parsed);

		if ((strpos($Text,'[/forum]') === false)) {
			$Text=$parsed;
			return;
		}


		if ($observer) {
			if ($connected) {
				$parsed = preg_replace("/\[forum\.observerconnected\=1\](.*?)\[\/forum\]/ism", '$1', $parsed);
				$parsed = preg_replace("/\[forum\.observerconnected\=0\](.*?)\[\/forum\]/ism", '', $parsed);
			} else {
				$parsed = preg_replace("/\[forum\.observerconnected\=1\](.*?)\[\/forum\]/ism", '', $parsed);
				$parsed = preg_replace("/\[forum\.observerconnected\=0\](.*?)\[\/forum\]/ism", '$1', $parsed);
			}



			if (!$connected) {
				$parsed = preg_replace("/\[forum\.observer\.myperm\=(.*?)](.*?)\[\/forum\]/ism",'',$parsed);
				$parsed = preg_replace("/\[forum\.observer\.nomyperm\=(.*?)](.*?)\[\/forum\]/ism",'',$parsed);
				$parsed = preg_replace("/\[forum\.observer\.theirperm\=(.*?)](.*?)\[\/forum\]/ism",'',$parsed);
				$parsed = preg_replace("/\[forum\.observer\.notheirperm\=(.*?)](.*?)\[\/forum\]/ism",'',$parsed);
				$Text = $parsed;
				return;
			}

			//MY_PERM
			$validperms = ['post_wall','send_stream','post_comments'];
			foreach ($validperms as $perm) {
				if (perm_is_allowed($uid,$observerhash,$perm) || $observerself) {
					$parsed = preg_replace("/\[forum\.observer\.myperm\=".$perm."\](.*?)\[\/forum\]/ism", '$1', $parsed);
					$parsed = preg_replace("/\[forum\.observer\.nomyperm\=".$perm."\](.*?)\[\/forum\]/ism", '', $parsed);
				} else {
					$parsed = preg_replace("/\[forum\.observer\.myperm\=".$perm."\](.*?)\[\/forum\]/ism", '', $parsed);
					$parsed = preg_replace("/\[forum\.observer\.nomyperm\=".$perm."\](.*?)\[\/forum\]/ism", '$1', $parsed);
				}
			}

			//THEIR_PERM
			$validperms = ['post_wall','send_stream','post_comments'];
			foreach ($validperms as $perm) {
				if (intval(get_abconfig($uid,$observerhash,'their_perms',$perm,0)) || $observerself) {
					$parsed = preg_replace("/\[forum\.observer\.theirperm\=".$perm."\](.*?)\[\/forum\]/ism", '$1', $parsed);
					$parsed = preg_replace("/\[forum\.observer\.notheirperm\=".$perm."\](.*?)\[\/forum\]/ism", '', $parsed);
				} else {
					$parsed = preg_replace("/\[forum\.observer\.theirperm\=".$perm."\](.*?)\[\/forum\]/ism", '', $parsed);
					$parsed = preg_replace("/\[forum\.observer\.notheirperm\=".$perm."\](.*?)\[\/forum\]/ism", '$1', $parsed);
				}
			}

		} 

		$Text = $parsed;
	}

	public static function hz_forum_item_store_before (&$hookinfo) {

	}


	public static function post_local (&$hookinfo) {

		$item=$hookinfo;
		$uid=$item['uid'];

		if (!Apps::addon_app_installed($uid,'forum')) {
			return;
		}

		$u = q("select * from channel left join xchan on channel_hash = xchan_hash where channel_id = %d limit 1",
                	intval($uid)
        	);

        	if(! $u)
                	return;

		$channel=$u[0];

                $edit = (($item['edited'] != $item['created']) || $item['item_deleted']);

                $arr = [];

                if ($edit) {

                        $r = q("select * from item where uid = %d and resource_id = '%s' and source_xchan = '%s' and resource_type = 'group_item' limit 1",
                                intval($channel['channel_id']),
                                dbesc($item['mid']),
                                dbesc($item['author_xchan'])
                        );

                        if ($r) {
				$olditem = $r[0];
				$oldtopic = IConfig::Get($olditem['id'],'forum','topicuuid','');
				if ($oldtopic) {
					IConfig::Set($item,'forum','topicuuid',$oldtopic,true);
				}
                        }
                        else {
                                logger('Forum post not found: ' . $item['mid']);
                                return;
                        }
                }

		else {
			if ($channel['channel_hash'] == $item['author_xchan']) {
				$topicuuid    = ((x($_REQUEST,'forumtopic')) ? $_REQUEST['forumtopic'] : '');
				$topicuuid = preg_replace('/[^a-zA-Z0-9\-\_]/','',$topicuuid);
				$topiclist = self::get_forumtopics($uid);
				if (isset($topiclist[$topicuuid])) {
					IConfig::Set($item, "forum", "topicuuid", $topicuuid, true);
					IConfig::Set($item,'forum','seen:'.$u['channel_hash'],'1',true);
				}
	
				$hookinfo = $item;
				return;
			}
		}

		$item['term'][] = ['ttype' => TERM_MENTION, 'url' => $channel['xchan_url'], 'term' => $channel['channel_address'].'@'.App::get_hostname()];

		$item['body'] .= "\n@![url=".$channel['xchan_url']."]".$channel['xchan_name'].'[/url]';
		$item['allow_cid'] = perms2str([$channel['channel_hash']]);

		$newhookinfo = $hookinfo;
		$newhookinfo = $item;
		$hookinfo = $newhookinfo;

		return;
	}

	public static function check_hashingroup ($uid,$hash,$gid) {

		$group=$gid;
		$channel = channelx_by_n($uid);
		if ($hash == $channel['channel_hash']) {
			return true;
		}

		if ($gid != "ChannelSettings" && $gid != "Anonymous" && $gid != '') {
			//NOTE: "Anonymous" posting not yet implemented.

			$ingroups=Group::containing($uid,$hash);
			$is_ingroup = false;

			$grouprec = Group::rec_byhash($uid,$gid);

			if ($grouprec) {
				$gid = $grouprec['id'];

				if (in_array($gid,$ingroups)) {
					$is_ingroup = true;
				}
			} else {

			}

			if (!$is_ingroup) {
				logger("Not INGROUP (Allowed: ".$group.")",LOGGER_DEBUG);
				return false;
			}
		}

		return true;
	}

	public static function hash_canpost($uid,$hash,$topicuuid) {

		$topiclist = self::get_forumtopics($uid);

		if (!isset($topiclist[$topicuuid])) {
			return false;
		}

		$topic = $topiclist[$topicuuid];

		if (isset($topic['post_allow_gid']) 
			&& $topic['post_allow_gid'] != ''
			&& $topic['post_allow_gid'] != 'ChannelSettings'
			) {
			if (!self::check_hashingroup ($uid,$hash,$topic['post_allow_gid'])) {

					return false;
			}
		}

		//Check Channel Settings
		if (!(perm_is_allowed($uid,$hash,'post_wall') || perm_is_allowed($uid,$hash,'tag_deliver'))) {
			return false;
		}

		return true;

	}

	public static function channel_apps(&$arr) {

		$uid=App::$profile_uid;
		if (!Apps::addon_app_installed($uid,'forum')) {
			return;
		}

		$channel = channelx_by_n($uid);
		$nickname = $channel['channel_address'];

		$newarr = $arr;
		$tabs = $arr['tabs'];

                $tabs[] = [
                        'label' => t('Forum'),
                        'url'   => z_root() . '/forum/' . $nickname,
                        'sel'   => ((argv(0) == 'forum') ? 'active' : ''),
                        'title' => t('Forum'),
                        'id'    => 'forum-tab',
                        'icon'  => 'commenting-o'
                ];

		$newarr['tabs'] = $tabs;
		$arr=$newarr;
		return;
        }


}
